      program run_sigma_ed
      dimension sp_f(3)

c electron kinematics 
      common/electron/eil,esc,theta_e
      
      pi = acos(-1.0)       
      pm = 0.938279         
      dm = 1.875628 
      dtr = pi/180.
      
c open input file      
      open(15, file='sigma_ed.kin', status = 'old')
c write header into output file
      write(12,*) "#! thr0[f,0]/ phr0[f,1]/ pr[f,2]/ ei[f,3]/ "//
     >     "th_e[f,4]/ q2[f,5]/ "//
     >     "x[f,6]/ plab[f,7]/ crs0[f,8]/ crs12[f,9]/ ratio[f,10]/ "
c number of kinematics
      read(15,*) nkin, iw
*     Initialization
      icon = 0
      write (6,*) "using wave function type : ", iw
      call sigma_ed(ei,q2,pr,thr,phir,ktf,sp_f,noff,npv,npro,icon,
     &     iv1,iv2,isd,iw,ipp,crs,ifl,q0)
      write (6,*) "initialize done"
      do nk = 1, nkin
         read(15,*) ei, pr, thr0, phir0, ktf, noff, npv, npro, icon,
     >        isd, ipp, iv1, iv2, q2       
c convert to radians         
         thr = thr0*dtr
         phir = phir0*dtr
*     
**************************************
*     INPUT   PARAMETERS   
**************************************
!      ei   = 11.00              ! initial electron energy
!      pr   = 0.1                ! recoil nucleon momentum GeV/C
!      thr  = 71.*pi/180.        ! polar angle of the recoil nucleon
!      phir = pi                 ! azimuthal angle of the recoil nucleon
!      ktf  = 1                  ! the knock-out nucleon is proton(1), neutron(-1)
!      noff = 1                  ! Off-Shell effects included (1) not included (0)
!      npv  = 0                  ! only pole term in FSI (0), pole+PV (1) 
!      npro = 1                  ! it calculates d\sigma/dE'_e d\Omega_e d\Omega_pf
!      icon = 0                  ! 0=calculate (initialization parameter)
!      isd  = 1                  ! spectator mechanism
!      ipp  = 0                  ! S and D component in deuteron
!      q2 =   4.0
      
*********************************************************************
*     fixing knock-out nucleons polarization axis along z
*********************************************************************
         sp_f(1) = 0.0; sp_f(2) = 0.0; sp_f(3) = 1.0
         icon_pwia=1                 ! PWIA
         call sigma_ed(ei,q2,pr,thr,phir,ktf,sp_f,
     &        noff,npv,npro,icon_pwia,
     &        iv1,iv2,isd,iw,ipp,crs0,ifl,q0)
         ! iv1 = 3                ! FSI with diffractive parameterization for f_nn
         ! iv2 = 10               ! Charge Exchange Gibbs_Luiseu parameterisation
         ! icon=12                !3           ! PWI+FSI !+CHEX
         write (26,*) "Kinematics : ", nk, " theta, phi = ", 
     >        thr0, phir0
         call flush
*     
**************************************
*     INPUT   PARAMETERS   
**************************************
!      ei   = 11.00              ! initial electron energy

         call sigma_ed(ei,q2,pr,thr,phir,ktf,sp_f,
     &        noff,npv,npro,icon,
     &        iv1,iv2,isd,iw,ipp,crs12,ifl,q0)
               
*     icon = 12
*     iv1  = 1
*     iv2  = 10021
*     call sigma_ed(ei,q2,pr,thr,phir,ktf,noff,npro,icon,
*     &                 iv1,iv2,isd,iw,ipp,crs,ifl,q0)
c     if(ifl.eq.1)goto 1      ! kinematically forbidden
         if(ifl .ne. 1) then
            x = q2/(2.0*pm*q0)
            ratio = 0
            if(crs0.ne.0.0)ratio=crs12/crs0
            
            tiv = 0.389385*1000.*1000.
            s = dm**2 + 2.0*dm*q0 - q2
            elab = (s-2.0*pm**2)/(2.0*pm)
            plab = sqrt(elab**2-pm**2)
            
            write(6,*)q2,pr,thr0,x,plab,crs0,crs12,ratio
            
            write(12,*)thr0, phir0, pr,ei, theta_e/dtr, q2,x,plab,
     >           crs0,crs12,ratio

         endif
      enddo
      end




